@@ -24,6 +24,8 @@
 #include <stdlib.h>
 #include <errno.h>
 
+#include "pango-markup.h"
+
 #include "pango-attributes.h"
 #include "pango-font.h"
 #include "pango-enum-types.h"
@@ -376,6 +378,9 @@ start_element_handler  (GMarkupParseContext *context,
       if (strcmp ("u", element_name) == 0)
 	parse_func = u_parse_func;
       break;
+
+    default:
+      break;
     }
 
   if (parse_func == NULL)
@@ -860,11 +865,29 @@ big_parse_func      (MarkupData            *md G_GNUC_UNUSED,
   return TRUE;
 }
 
+static gboolean
+parse_percentage (const char *input,
+                  double     *val)
+{
+  double v;
+  char *end;
+
+  v = g_ascii_strtod (input, &end);
+  if (errno == 0 && strcmp (end, "%") == 0 && v > 0)
+    {
+      *val = v;
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static gboolean
 parse_absolute_size (OpenTag               *tag,
-		     const char            *size)
+                     const char            *size)
 {
   SizeLevel level = Medium;
+  double val;
   double factor;
 
   if (strcmp (size, "xx-small") == 0)
@@ -881,6 +904,11 @@ parse_absolute_size (OpenTag               *tag,
     level = XLarge;
   else if (strcmp (size, "xx-large") == 0)
     level = XXLarge;
+  else if (parse_percentage (size, &val))
+    {
+      factor = val / 100.0;
+      goto done;
+    }
   else
     return FALSE;
 
@@ -888,6 +916,8 @@ parse_absolute_size (OpenTag               *tag,
    * but not to sizes created by any other tags
    */
   factor = scale_factor (level, 1.0);
+
+done:
   add_attribute (tag, pango_attr_scale_new (factor));
   if (tag)
     open_tag_set_absolute_font_scale (tag, factor);
@@ -951,6 +981,29 @@ span_parse_int (const char *attr_name,
   return TRUE;
 }
 
+static gboolean
+span_parse_float (const char  *attr_name,
+                  const char  *attr_val,
+                  double      *val,
+                  int          line_number,
+                  GError     **error)
+{
+  *val = g_ascii_strtod (attr_val, NULL);
+  if (errno != 0)
+    {
+      g_set_error (error,
+                   G_MARKUP_ERROR,
+                   G_MARKUP_ERROR_INVALID_CONTENT,
+                   _("Value of '%s' attribute on <span> tag "
+                     "on line %d could not be parsed; "
+                     "should be a number, not '%s'"),
+                   attr_name, line_number, attr_val);
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 static gboolean
 span_parse_boolean (const char *attr_name,
 		    const char *attr_val,
@@ -1108,6 +1161,35 @@ span_parse_flags (const char  *attr_name,
   return TRUE;
 }
 
+static gboolean
+parse_length (const char *attr_val,
+              int        *result)
+{
+  const char *attr;
+  int n;
+
+  attr = attr_val;
+  if (_pango_scan_int (&attr, &n) && *attr == '\0')
+    {
+      *result = n;
+      return TRUE;
+    }
+  else
+    {
+      double val;
+      char *end;
+
+      val = g_ascii_strtod (attr_val, &end);
+      if (errno == 0 && strcmp (end, "pt") == 0)
+        {
+          *result = val * PANGO_SCALE;
+          return TRUE;
+        }
+    }
+
+  return FALSE;
+}
+
 static gboolean
 span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 		     OpenTag               *tag,
@@ -1146,6 +1228,8 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
   const char *allow_breaks = NULL;
   const char *insert_hyphens = NULL;
   const char *show = NULL;
+  const char *line_height = NULL;
+  const char *text_transform = NULL;
 
   g_markup_parse_context_get_position (context,
 				       &line_number, &char_number);
@@ -1214,6 +1298,9 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 	CHECK_ATTRIBUTE (strikethrough_color);
 	CHECK_ATTRIBUTE (style);
 	break;
+      case 't':
+        CHECK_ATTRIBUTE (text_transform);
+        break;
       case 'g':
 	CHECK_ATTRIBUTE (gravity);
 	CHECK_ATTRIBUTE (gravity_hint);
@@ -1224,6 +1311,7 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
       case 'l':
 	CHECK_ATTRIBUTE (lang);
 	CHECK_ATTRIBUTE (letter_spacing);
+        CHECK_ATTRIBUTE (line_height);
 	break;
       case 'o':
 	CHECK_ATTRIBUTE (overline);
@@ -1280,27 +1368,14 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 
   if (G_UNLIKELY (size))
     {
-      if (g_ascii_isdigit (*size))
-	{
-	  const char *end;
-	  gint n;
+      int n;
 
-	  if ((end = size, !_pango_scan_int (&end, &n)) || *end != '\0' || n < 0)
-	    {
-	      g_set_error (error,
-			   G_MARKUP_ERROR,
-			   G_MARKUP_ERROR_INVALID_CONTENT,
-			   _("Value of 'size' attribute on <span> tag on line %d "
-			     "could not be parsed; should be an integer no more than %d,"
-			     " or a string such as 'small', not '%s'"),
-			   line_number, INT_MAX, size);
-	      goto error;
-	    }
-
-	  add_attribute (tag, pango_attr_size_new (n));
-	  if (tag)
-	    open_tag_set_absolute_font_size (tag, n);
-	}
+      if (parse_length (size, &n) && n > 0)
+        {
+          add_attribute (tag, pango_attr_size_new (n));
+          if (tag)
+            open_tag_set_absolute_font_size (tag, n);
+        }
       else if (strcmp (size, "smaller") == 0)
 	{
 	  if (tag)
@@ -1508,7 +1583,7 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 	  g_set_error (error,
 		       G_MARKUP_ERROR,
 		       G_MARKUP_ERROR_INVALID_CONTENT,
-		       _("'%s' is not a valid value for the 'stretch' "
+		       _("'%s' is not a valid value for the 'gravity' "
 			 "attribute on <span> tag, line %d; valid "
 			 "values are for example 'south', 'east', "
 			 "'north', 'west'"),
@@ -1569,12 +1644,31 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
       add_attribute (tag, pango_attr_show_new (flags));
     }
 
+  if (G_UNLIKELY (text_transform))
+    {
+      PangoTextTransform tf;
+
+      if (!span_parse_enum ("text_transform", text_transform, PANGO_TYPE_TEXT_TRANSFORM, (int*)(void*)&tf, line_number, error))
+	goto error;
+
+      add_attribute (tag, pango_attr_text_transform_new (tf));
+    }
+
   if (G_UNLIKELY (rise))
     {
       gint n = 0;
 
-      if (!span_parse_int ("rise", rise, &n, line_number, error))
-	goto error;
+      if (!parse_length (rise, &n))
+        {
+          g_set_error (error,
+                       G_MARKUP_ERROR,
+                       G_MARKUP_ERROR_INVALID_CONTENT,
+                       _("Value of 'rise' attribute on <span> tag on line %d "
+                         "could not be parsed; should be an integer, or a "
+                         "string such as '5.5pt', not '%s'"),
+                       line_number, rise);
+          goto error;
+        }
 
       add_attribute (tag, pango_attr_rise_new (n));
     }
@@ -1589,6 +1683,19 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
       add_attribute (tag, pango_attr_letter_spacing_new (n));
     }
 
+  if (G_UNLIKELY (line_height))
+    {
+      double f = 0;
+
+      if (!span_parse_float ("line_height", line_height, &f, line_number, error))
+        goto error;
+
+      if (f > 1024.0 && strchr (line_height, '.') == 0)
+        add_attribute (tag, pango_attr_line_height_new_absolute ((int)f));
+      else
+        add_attribute (tag, pango_attr_line_height_new (f));
+    }
+
   if (G_UNLIKELY (lang))
     {
       add_attribute (tag,
