@@ -250,6 +250,8 @@ draw_underline (PangoRenderer *renderer,
                                            rect->width,
                                            3 * rect->height);
       break;
+    default:
+      break;
     }
 }
 
@@ -274,6 +276,8 @@ draw_overline (PangoRenderer *renderer,
                                      rect->width,
                                      rect->height);
       break;
+    default:
+      break;
     }
 }
 
@@ -284,7 +288,7 @@ draw_strikethrough (PangoRenderer *renderer,
   PangoRectangle *rect = &state->strikethrough_rect;
   int num_glyphs = state->strikethrough_glyphs;
 
-  if (state->strikethrough)
+  if (state->strikethrough && num_glyphs > 0)
     pango_renderer_draw_rectangle (renderer,
                                    PANGO_RENDER_PART_STRIKETHROUGH,
                                    rect->x,
@@ -388,6 +392,8 @@ add_underline (PangoRenderer    *renderer,
           current_rect->height = new_rect.height;
         }
       break;
+    default:
+      break;
     }
 
   if (renderer->underline == state->underline &&
@@ -439,6 +445,8 @@ add_overline (PangoRenderer    *renderer,
           current_rect->height = new_rect.height;
         }
       break;
+    default:
+      break;
     }
 
   if (renderer->priv->overline == state->overline &&
