@@ -194,15 +194,22 @@ dump_directions (PangoLayout *layout, GString *string)
 static void
 dump_cursor_positions (PangoLayout *layout, GString *string)
 {
+  const char *text;
   int index, trailing;
 
+  text = pango_layout_get_text (layout);
+
   index = 0;
   trailing = 0;
 
   while (index < G_MAXINT)
     {
       g_string_append_printf (string, "%d(%d) ", index, trailing);
-      pango_layout_move_cursor_visually (layout, TRUE, index, trailing, 1, &index, &trailing);
+
+      while (trailing--)
+        index = g_utf8_next_char (text + index) - text;
+
+      pango_layout_move_cursor_visually (layout, TRUE, index, 0, 1, &index, &trailing);
     }
 
   g_string_append (string, "\n");
@@ -221,6 +228,7 @@ typedef struct {
   gboolean auto_dir;
   gboolean single_paragraph;
   PangoTabArray *tabs;
+  PangoGravity gravity;
 } LayoutParams;
 
 static void
@@ -238,6 +246,7 @@ init_params (LayoutParams *params)
   params->auto_dir = TRUE;
   params->single_paragraph = FALSE;
   params->tabs = NULL;
+  params->gravity = PANGO_GRAVITY_AUTO;
 }
 
 static void
@@ -326,6 +335,16 @@ parse_params (const char   *str,
             }
           g_strfreev (str3);
         }
+      else if (strcmp (str2[0], "gravity") == 0)
+        {
+          eclass = g_type_class_ref (PANGO_TYPE_GRAVITY);
+          ev = g_enum_get_value_by_name (eclass, str2[1]);
+          if (!ev)
+            ev = g_enum_get_value_by_nick (eclass, str2[1]);
+          if (ev)
+            params->gravity = ev->value;
+          g_type_class_unref (eclass);
+        }
 
       g_strfreev (str2);
     }
@@ -425,6 +444,8 @@ test_file (const char *filename, GString *string)
 
   parse_params (contents, &params);
 
+  pango_context_set_base_gravity (context, params.gravity);
+
   pango_layout_set_width (layout, params.width > 0 ? params.width * PANGO_SCALE : -1);
   pango_layout_set_height (layout, params.height > 0 ? params.height * PANGO_SCALE : params.height);
   pango_layout_set_indent (layout, params.indent * PANGO_SCALE);
@@ -458,8 +479,8 @@ test_file (const char *filename, GString *string)
 
   g_assert_cmpint (pango_layout_get_character_count (layout), ==, g_utf8_strlen (pango_layout_get_text (layout), -1));
 
-  /* Some checks on extents - we have to be careful here, since we
-   * don't want to depend on font metrics.
+  /* Some checks on extents - we have to be careful here,
+   * since we don't want to depend on font metrics.
    */
   pango_layout_get_size (layout, &width, &height);
   pango_layout_get_extents (layout, &ink_rect, &logical_rect);
@@ -480,15 +501,12 @@ test_file (const char *filename, GString *string)
   for (l = lines; l; l = l->next)
     {
       PangoLayoutLine *line = l->data;
-      int line_height, line_width;
+      int line_width;
       int line_x;
       PangoRectangle line_ink, line_logical;
       PangoRectangle line_ink1, line_logical1;
       gboolean done;
 
-      pango_layout_line_get_height (line, &line_height);
-      g_assert_cmpint (line_height, <=, height);
-
       pango_layout_line_get_extents (line, &line_ink, &line_logical);
       line_x = line_logical.x;
       line_width = line_logical.width;
@@ -514,7 +532,7 @@ test_file (const char *filename, GString *string)
       while (!done && pango_layout_iter_get_line_readonly (iter) == line)
         {
           int prev_index, index, next_index;
-          int x, index2, trailing;
+          int x;
           int *ranges;
           int n_ranges;
           gboolean found_range;
@@ -527,20 +545,9 @@ test_file (const char *filename, GString *string)
             done = TRUE;
 
           pango_layout_line_index_to_x (line, index, 0, &x);
-          pango_layout_line_x_to_index (line, x, &index2, &trailing);
-
-#if 0
-          /* FIXME: why doesn't this hold true? */
-          g_assert_cmpint (index2, ==, index);
-          g_assert_cmpint (trailing, ==, 0);
-#endif
-
           g_assert_cmpint (0, <=, x);
           g_assert_cmpint (x, <=, line_width);
 
-          g_assert_cmpint (line->start_index, <=, index2);
-          g_assert_cmpint (index2, <=, line->start_index + line->length);
-
           if (!run)
             break;
 
