@@ -306,6 +306,189 @@ test_fallback_shape (void)
   g_object_unref (context);
 }
 
+/* https://bugzilla.gnome.org/show_bug.cgi?id=547303 */
+static void
+test_get_cursor_crash (void)
+{
+  PangoContext *context;
+  PangoLayout *layout;
+  int i;
+
+  const char *string = "foo\n\rbar\r\nbaz\n\nqux\n\n..";
+
+  context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+
+  layout = pango_layout_new (context);
+
+  pango_layout_set_text (layout, string, -1);
+
+  for (i = 0; string[i]; i++)
+    {
+      PangoRectangle rectA, rectB;
+
+      pango_layout_get_cursor_pos (layout, i, &rectA, &rectB);
+      g_assert_cmpint (rectA.x, ==, rectB.x);
+    }
+
+  g_object_unref (layout);
+  g_object_unref (context);
+}
+
+/* Test that get_cursor returns split cursors in the
+ * expected situations. In particular, this was broken
+ * at the end of the string here.
+ */
+static void
+test_get_cursor (void)
+{
+  const char *text = "abאב";
+  PangoContext *context;
+  PangoLayout *layout;
+  PangoRectangle strong, weak;
+
+  context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+
+  layout = pango_layout_new (context);
+  pango_layout_set_text (layout, text, -1);
+
+  pango_layout_get_cursor_pos (layout, 0, &strong, &weak);
+  g_assert_cmpint (strong.x, ==, weak.x);
+
+  pango_layout_get_cursor_pos (layout, 1, &strong, &weak);
+  g_assert_cmpint (strong.x, ==, weak.x);
+
+  pango_layout_get_cursor_pos (layout, 2, &strong, &weak);
+  g_assert_cmpint (strong.x, !=, weak.x);
+
+  pango_layout_get_cursor_pos (layout, 4, &strong, &weak);
+  g_assert_cmpint (strong.x, ==, weak.x);
+
+  pango_layout_get_cursor_pos (layout, 6, &strong, &weak);
+  g_assert_cmpint (strong.x, !=, weak.x);
+
+  g_object_unref (layout);
+  g_object_unref (context);
+}
+
+static void
+test_index_to_x (void)
+{
+  PangoContext *context;
+  const char *tests[] = {
+    "ac­ual­ly", // soft hyphen
+    "ac​ual​ly", // zero-width space
+  };
+
+  context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+
+  for (int i = 0; i < G_N_ELEMENTS (tests); i++)
+    {
+      PangoLayout *layout;
+      const char *text;
+      const char *p;
+
+      layout = pango_layout_new (context);
+      pango_layout_set_text (layout, tests[i], -1);
+      text = pango_layout_get_text (layout);
+
+      for (p = text; *p; p = g_utf8_next_char (p))
+        {
+          int index = p - text;
+          int l;
+          PangoLayoutLine *line;
+          int x;
+          int index2, trailing;
+          gunichar ch;
+
+          ch = g_utf8_get_char (p);
+
+          pango_layout_index_to_line_x (layout, index, FALSE, &l, NULL);
+          line = pango_layout_get_line (layout, l);
+          g_assert_nonnull (line);
+
+          pango_layout_line_index_to_x (line, index, 0, &x);
+          pango_layout_line_x_to_index (line, x, &index2, &trailing);
+          if (!pango_is_zero_width (ch))
+            g_assert_cmpint (index, ==, index2);
+        }
+
+      g_object_unref (layout);
+    }
+
+  g_object_unref (context);
+}
+
+static gboolean
+pango_rectangle_contains (const PangoRectangle *r1,
+                          const PangoRectangle *r2)
+{
+  return r2->x >= r1->x &&
+         r2->y >= r1->y &&
+         r2->x + r2->width <= r1->x + r1->width &&
+         r2->y + r2->height <= r1->y + r1->height;
+}
+
+static void
+test_extents (void)
+{
+  PangoContext *context;
+  const char *tests[] = {
+    "Some long text that has multiple lines that are wrapped by Pango."
+  };
+
+  context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+
+  for (int i = 0; i < G_N_ELEMENTS (tests); i++)
+    {
+      PangoLayout *layout;
+      PangoLayoutIter *iter;
+      PangoRectangle layout_extents;
+      PangoRectangle line_extents;
+      PangoRectangle run_extents;
+      PangoRectangle cluster_extents;
+      PangoRectangle char_extents;
+      PangoRectangle pos;
+
+      layout = pango_layout_new (context);
+      pango_layout_set_text (layout, tests[i], -1);
+      pango_layout_set_width (layout, 60 * PANGO_SCALE);
+
+      pango_layout_get_extents (layout, NULL, &layout_extents);
+
+      iter = pango_layout_get_iter (layout);
+
+      do
+        {
+          pango_layout_iter_get_line_extents (iter, NULL, &line_extents);
+          pango_layout_iter_get_run_extents (iter, NULL, &run_extents);
+          pango_layout_iter_get_cluster_extents (iter, NULL, &cluster_extents);
+          pango_layout_iter_get_char_extents (iter, &char_extents);
+
+          pango_layout_index_to_pos (layout,
+                                     pango_layout_iter_get_index (iter),
+                                     &pos);
+          if (pos.width < 0)
+            {
+              pos.x += pos.width;
+              pos.width = - pos.width;
+            }
+
+          g_assert_true (pango_rectangle_contains (&layout_extents, &line_extents));
+          g_assert_true (pango_rectangle_contains (&line_extents, &run_extents));
+          g_assert_true (pango_rectangle_contains (&run_extents, &cluster_extents));
+          g_assert_true (pango_rectangle_contains (&cluster_extents, &char_extents));
+
+          g_assert_true (pango_rectangle_contains (&line_extents, &pos));
+        }
+      while (pango_layout_iter_next_char (iter));
+
+      pango_layout_iter_free (iter);
+      g_object_unref (layout);
+    }
+
+  g_object_unref (context);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -318,8 +501,8 @@ main (int argc, char *argv[])
   g_test_add_func ("/language/emoji-crash", test_language_emoji_crash);
   g_test_add_func ("/layout/line-height", test_line_height);
   g_test_add_func ("/attr-list/update", test_attr_list_update);
-  g_test_add_func ("/version-info", test_version_info);
-  g_test_add_func ("/is-zerowidth", test_is_zero_width);
+  g_test_add_func ("/misc/version-info", test_version_info);
+  g_test_add_func ("/misc/is-zerowidth", test_is_zero_width);
   g_test_add_func ("/gravity/to-rotation", test_gravity_to_rotation);
   g_test_add_func ("/gravity/from-matrix", test_gravity_from_matrix);
   g_test_add_func ("/gravity/for-script", test_gravity_for_script);
@@ -327,6 +510,10 @@ main (int argc, char *argv[])
 #ifdef HAVE_CAIRO_FREETYPE
   g_test_add_func ("/language/to-tag", test_language_to_tag);
 #endif
+  g_test_add_func ("/bidi/get-cursor-crash", test_get_cursor_crash);
+  g_test_add_func ("/bidi/get-cursor", test_get_cursor);
+  g_test_add_func ("/layout/index-to-x", test_index_to_x);
+  g_test_add_func ("/layout/extents", test_extents);
 
   return g_test_run ();
 }
